import numpy as np
import math
import sys
import io

# Module to write Desmos console code to render a chemical process block flow diagram (BFD) or flowchart
# The output of this code should be pasted into an html file

np.set_printoptions(legacy='1.25')

def diagram(exps,blocks,streams,labels,size=[1000,400],bor=[-10,10,-10,10],font=0.8,tip=0.3,
            ang=math.pi/12,tipv=0.3,angv=math.pi/12,display=''):
    if display == 'string':
        original_screen = sys.stdout
        diagram_html = io.StringIO()
        sys.stdout = diagram_html
    
    
    hsize = size[0] # horizontal size of graph
    vsize = size[1] # vertical size of graph
    
    lbor = bor[0] # left border (x-coordinate)
    rbor = bor[1] # right border (x-coordinate)
    bbor = bor[2] # bottom border (y-coordinate)
    tbor = bor[3] # top border (y-coordinate)
 
    indent = '  ' # indentation for html file
    
    # Introductory commands
    
    if display != 'string':
        print('Create an html file from the entire output or copy the script section into the Desmos console with Ctrl+Shift+J')
        print()
    
    print('<script src="https://www.desmos.com/api/v1.9/calculator.js?apiKey=dcb31709b452b1cf9dc26972add0fda6"></script>')
    
    s = '<div id="calculator" style="width: hsizepx; height: vsizepx;"></div>'
    s = s.replace('hsize',str(hsize))
    s = s.replace('vsize',str(vsize))
    print(s) # dimensions
    
    print('<script>') 
    
    print(indent + "var elt = document.getElementById('calculator');")
    print(indent + 'var Calc = Desmos.GraphingCalculator(elt);')
    print()
    
    # Expressions
    
    if exps != [[]]:
        lines = len(exps)
        for i in np.arange(lines):
            s0 = indent + 'Calc.setExpression({ id: "name", type: "expression", latex: "explatex", color: "hue" })'
            
            s = s0.replace('name',str(exps[i][0]))
            s = s.replace('explatex',str(exps[i][1]))
            s = s.replace('hue',str(exps[i][2]))
            print(s)
        print()
    
    # Blocks
       
    if blocks != [[]]:
        lines = len(blocks)
        for i in np.arange(lines):
            # Draw blocks as polygons
            
            s0 = indent + 'Calc.setExpression({ id:"name", type: "table", columns: [ { latex: "XX", values: xval }, { latex: "YY", values: yval, points: false } ] })'
            
            s = s0.replace('name',str(blocks[i][0]))
            s = s.replace('XX',str(blocks[i][1]))
            s = s.replace('YY',str(blocks[i][2]))
            
            s = s.replace('xval',np.array2string(np.array(blocks[i][3])[0,:],separator=", "))
            s = s.replace('yval',np.array2string(np.array(blocks[i][3])[1,:],separator=", "))
            print(s)
            
            s0 = indent + 'Calc.setExpression({ id:"blockname", latex:"\\\\operatorname{polygon}\\\\left("XX","YY"\\\\right)", color:"hue", fillOpacity: dense, lines:true })'
    
            s = s0.replace('blockname',str(blocks[i][0]) + 'polygon')
            s = s.replace('"XX"',str(blocks[i][1]))
            s = s.replace('"YY"',str(blocks[i][2]))
            s = s.replace('hue',str(blocks[i][4]))
            s = s.replace('dense',str(blocks[i][5]))
            print(s)       
        print()
        
    # Streams
        
    if streams != [[]]:
        lines = len(streams)
        for i in np.arange(lines):
            # Draw streams consisting of one or more segments
            
            s0 = indent + 'Calc.setExpression({ id:"name", latex:"coordinates", color:"hue", points:false, lines:true, lineWidth:thickness })'
            
            s = s0.replace('name',str(streams[i][0]))
            
            st = str(streams[i][1])
            st = st.replace('[[','[')
            st = st.replace(']]',']')
            s = s.replace('coordinates','(' + st + ')')
            
            s = s.replace('hue',str(streams[i][2]))
            s = s.replace('thickness',str(streams[i][3]))
            print(s)
            
            # Draw arrows if specified for particular streams
            
            if len(streams[i]) > 4: # an arrow direction is specified for a stream
                XY = np.array(streams[i][1]) # extract X and Y coordinates
                ydim = XY.shape[1] # y-dimension of coordinate matrix
                
                if streams[i][4] != '': # right arrow requested
                    xy = XY[:,ydim-1] # obtain last point; arrowhead is always drawn at this point
                    x0 = xy[0]; y0 = xy[1] # obtain x- and y-coordinates of this point
                    
                    if streams[i][4] == 'left':
                        xa = x0 + tip*math.cos(ang); ya = y0 + tip*math.sin(ang)
                        xb = x0 + tip*math.cos(ang); yb = y0 - tip*math.sin(ang)
                    elif streams[i][4] == 'right':
                        xa = x0 - tip*math.cos(ang); ya = y0 + tip*math.sin(ang)
                        xb = x0 - tip*math.cos(ang); yb = y0 - tip*math.sin(ang)
                    elif streams[i][4] == 'up':
                        xa = x0 + tipv*math.cos(angv); ya = y0 - tipv*math.sin(angv)
                        xb = x0 - tipv*math.cos(angv); yb = y0 - tipv*math.sin(angv) 
                    elif streams[i][4] == 'down':
                        xa = x0 + tipv*math.cos(angv); ya = y0 + tipv*math.sin(angv)
                        xb = x0 - tipv*math.cos(angv); yb = y0 + tipv*math.sin(angv)                    
                    
                    xyarr = [[xa,x0,xb],[ya,y0,yb]]
                    
                    s = s0.replace('name',str(streams[i][0]) + 'arr')
                    st = str(xyarr)
                    st = st.replace('[[','[')
                    st = st.replace(']]',']')
                    s = s.replace('coordinates','(' + st + ')')
                    
                    s = s.replace('hue',str(streams[i][2]))
                    s = s.replace('thickness',str(streams[i][3]))
                    print(s)
        print()
    
    # Labels
    
    if labels != [[]]:  
        lines = len(labels)
        for i in np.arange(lines):
            # Place labels
            
            s0 = indent + 'Calc.setExpression({ id:"name", latex:"coordinates", color:"hue", hidden:true, showLabel:true, labelOrientation:"direction", labelSize:font, label:"labeltext" })'
            
            s = s0.replace('name',str(labels[i][0]))
            
            st = str(labels[i][1])
            st = st.replace('[','')
            st = st.replace(']','')
            s = s.replace('coordinates','(' + st + ')')
            
            s = s.replace('hue',str(labels[i][3]))
            s = s.replace('font','"' + str(labels[i][4]) + '"')
            s = s.replace('direction',str(labels[i][5]))
            s = s.replace('labeltext',str(labels[i][2]))
            print(s)
            
        print()
        
    # Concluding commands
    
    s = indent + 'Calc.setMathBounds({ left: lbor, right: rbor, bottom: bbor, top: tbor })'
    s = s.replace('lbor',str(lbor))
    s = s.replace('rbor',str(rbor))
    s = s.replace('tbor',str(tbor))
    s = s.replace('bbor',str(bbor))
    print(s) # x and y ranges of graph
    
    print(indent + 'Calc.updateSettings({ showGrid: false, showXAxis: false, showYAxis: false, lockViewport: true, expressionsCollapsed: true })')
    print('</script>')
    
    if display == 'string':
        sys.stdout = original_screen
        return diagram_html



import numpy as np
import pandas as pd

# Module to write Desmos code for fill-in-the-table question

def tbl (file,tola=1.0,tolf=0.01,feedback='individual',samesheet=0,sheet='',skiprows=-1,nrows=-1,usecols="",lock=True):
    
    # Data, mode and format tables can be suppied in two ways
    
    # One method is to use three Excel sheets named data, mode and format
    # In this method, the sheets should only contain the relevant tables
    # For this method, samesheet is 0
    
    # Another method is to use any three non-overlapping regions in a single Excel sheet
    # This should be the only sheet in the Excel file
    # For this method, samesheet = 1f
    # The vectors skiprows, nrows and usecols should be 3-component vectors
    # The three components should correspond to data, mode and format

    import sys
    sys.stdin.reconfigure(encoding='utf-8')
    sys.stdout.reconfigure(encoding='utf-8')
    
    if samesheet == 0:
        datafrm = pd.read_excel(file, sheet_name='data')
    else:
        datafrm = pd.read_excel(file, sheet_name=sheet, skiprows=skiprows[0], nrows=nrows[0], usecols=usecols[0])    
    data = datafrm.to_numpy()
    
    if samesheet == 0:
        mode = pd.read_excel(file, sheet_name='mode').to_numpy()
    else:
        mode = pd.read_excel(file, sheet_name=sheet, skiprows=skiprows[1], nrows=nrows[1], usecols=usecols[1]).to_numpy()
    
    if samesheet == 0:
        frmt = pd.read_excel(file, sheet_name='format').to_numpy()
    else:
        frmt = pd.read_excel(file, sheet_name=sheet, skiprows=skiprows[2], nrows=nrows[2], usecols=usecols[2]).to_numpy()
    
    rows = np.size(data,0)
    cols = np.size(data,1)
    rr = int(np.floor(np.log(rows)/np.log(10))+1) # zero-fill variable names based on total rows
    cc = int(np.floor(np.log(cols)/np.log(10))+1) # zero-fill variable names based on total columns
    
    cor = ''; count = 0 # string to collect correctness variables for entire table
    
    for j in np.arange(0,len(datafrm.columns)): # print headers
        cellij = '(' + str(0) + ',' + str(j+1) + ')'
        print('cellContent' + cellij + ': "' + datafrm.columns[j] + '"')
    print()
    
    expcount = 0 # number of expressions without = sign
    varmax = 0 # maxnumber of variables in a single expression without = sign
            
    for i in np.arange(0,rows):
        if feedback == 'row':
            cori = ''; counti = 0; # string to collect correctness variables for current row
        
        for j in np.arange(0,cols):
            cellij = '(' + str(i+1) + ',' + str(j+1) + ')'
            
            if mode[i,j] == 'display': # entries displayed verbatim
                if frmt[i,j] == 'text' or frmt[i,j] == 'expression' or frmt[i,j] == 'equation':
                    print('cellContent' + cellij + ': "' + data[i,j] + '"')
                    print()
                if frmt[i,j] == 'exact' or frmt[i,j] == 'number':
                    print('cellContent' + cellij + ': `' + str(data[i,j]) + '`')
                
            if mode[i,j] == 'test': # entries tested for
                if frmt[i,j] == 'text' or frmt[i,j] == 'exact' or frmt[i,j] == 'expression' or frmt[i,j] == 'equation' or frmt[i,j] == 'number':
                    ij = str(i+1).zfill(rr) + str(j+1).zfill(cc)
                    
                    if frmt[i,j] == 'text' and type(data[i,j]) == str:
                        print('cor' + ij + ' = when this.cellContent' + cellij + ' = "' + data[i,j] + '" true otherwise false')
                    elif frmt[i,j] == 'text' and (type(data[i,j]) == int or type(data[i,j]) == float):
                        print('cor' + ij + ' = when this.cellContent' + cellij + ' = "' + str(data[i,j]) + '" true otherwise false')
                    elif frmt[i,j] == 'exact' and (type(data[i,j]) == int or type(data[i,j]) == float):
                            print('cor' + ij + ' = when this.cellNumericValue' + cellij + ' = ' + str(data[i,j]) + ' true otherwise false')
                    elif frmt[i,j] == 'exact' and type(data[i,j]) == str: # used for variable names, e.g., row headers in a simplex tableau
                            print('cor' + ij + ' = when this.cellContent' + cellij + ' = `' + data[i,j] + '` true otherwise false')
                            
                    if frmt[i,j] == 'number': # number with tolerance range
                        dij = 'd' + ij
                        daij = 'a' + ij
                        fij = 'f' + ij
                        faij = 'fa' + ij
                        
                        print(dij + ' = simpleFunction("x-y","x","y").evaluateAt(this.cellNumericValue' + cellij + ',' + str(data[i,j]) + ')')
                        print(daij + ' = when ' + dij + ' < 0 simpleFunction("-x","x").evaluateAt(' + dij + ') otherwise ' + dij)
                        print(fij + ' = simpleFunction("x/y","x","y").evaluateAt(' + daij + ',' + str(data[i,j]) + ')')
                        print(faij + ' = when ' + fij + ' < 0 simpleFunction("-x","x").evaluateAt(' + fij + ') otherwise ' + fij)
                        print('cor' + ij + ' = when ' + daij + ' < ' + str(tola) + ' or ' + faij + ' < ' + str(tolf) + ' true otherwise false')                     
                    
                    if frmt[i,j] == 'expression': # expression without equal sign
                        expcount += 1
                        
                        dataij = data[i,j].replace('[','').replace(']]','').split('],')

                        vars = int(dataij[2])
                        if vars > varmax:
                            varmax = vars
                        
                        cc = int(np.floor(np.log(vars)/np.log(10))+1) # zero-fill variable names based on total variables
                        varrnd = ['var' + str(j+1).zfill(cc) for j in np.arange(0,vars)]
                        varrndlist = str(varrnd).replace('[','').replace(']','').replace("'",'')
                        
                        print('exp' + ij + ' = ' + dataij[0])
                        
                        fij = 'f' + ij
                        gij = 'g' + ij
                        dij = 'd' + ij
                        aij = 'a' + ij
                        
                        print(fij + ' = simpleFunction(exp' + ij + ',' + dataij[1] + ').evaluateAt(' + varrndlist + ')')
                        print(gij + ' = simpleFunction(this.cellContent' + cellij + ',' + dataij[1] + ').evaluateAt(' + varrndlist + ')')
                        print(dij + ' = simpleFunction("x-y","x","y").evaluateAt(' + fij + ',' + gij + ')')
                        print(aij + ' = when ' + dij + ' < 0 simpleFunction("-x","x").evaluateAt(' + dij + ') otherwise ' + dij)
                        print('cor' + ij + ' = when ' + aij + ' < tola true otherwise false')
                    
                    if frmt[i,j] == 'equation': # equation
                        dataij = data[i,j].replace('[','').replace(']]','').split('],')
                        print('eq' + ij + ' = ' + dataij[0])
                        
                        dij = 'd' + ij
                        aij = 'a' + ij
                        print(dij + ' = parseEquation(this.cellContent' + cellij + ').differenceFunction(' + dataij[1] + ').evaluateAt(' + dataij[2] + ')')
                        print(aij + ' = when ' + dij + ' < 0 simpleFunction("-x","x").evaluateAt(' + dij + ') otherwise ' + dij)
                        print('cor' + ij + ' = when ' + aij + ' < tola true otherwise false')
                        
                    if feedback == 'individual': # feedback requested for individual cells (default option)
                        print('cellSuffix' + cellij + ': when cor' + ij + ' "✅" otherwise "❌"')
                    
                    if lock == True:
                        print('cellEditable' + cellij + ': when submit.pressCount <= 1 true otherwise false')
                    else:
                        print('# cellEditable' + cellij + ': when submit.pressCount <= 1 true otherwise false')
                    
                    if count == 0:
                        cor += 'cor' + ij
                    else:
                        cor += ' and cor' + ij                           
                    count += 1
                    
                    if feedback == 'row':
                        if counti == 0:
                            cori += 'cor' + ij
                        else:
                            cori += ' and cor' + ij
                        counti += 1
                    print()
        if feedback == 'row':
            cellij1 = '(' + str(i+1) + ',' + str(j+2) + ')'
            print('cellContent' + cellij1 + ': when ' + cori + ' "✅" otherwise "❌"')
            print()
        
    if expcount > 0:
        print('tola = ' + str(tola))
        print()
        print('ran = randomGenerator()')    
    
        cc = int(np.floor(np.log(varmax)/np.log(10))+1) # zero-fill variable names based on total variables
        varrnd = ['var' + str(j+1).zfill(cc) for j in np.arange(0,varmax)]
        for j in np.arange(0,varmax):
            print(varrnd[j] + ' = ran.float(-999,-100)')
        
    print('correct: ' + cor)



import numpy as np

# Module to write Desmos code for checking inequalities

def inequality (names,eqns,variables,valueseq,valuesineq,strict,thr=0.01,inc=[[]],lock=True):
    # The vectors names and ineqn must be of the same size
    # The vectors variables must be of the same size as the columns of valueseq or valuesineq
    # Each inequality is tested a different point
    # This differs from equation, which tests all equations at the same point
    # Each inequality is first tested at its boundary and then at an interior feasible point
    # Inequalities have to be matched exactly; there is no option of inexact matching
    #   the incidence matrix inc must be sized with inequalities as rows and variables as columns
    #   this matrix must contain only 0s and 1s
    
    import sys
    sys.stdin.reconfigure(encoding='utf-8')
    sys.stdout.reconfigure(encoding='utf-8')

    rows = np.size(names,0)
    cols = np.size(variables,0)
    
    cc = int(np.floor(np.log(cols)/np.log(10))+1) # zero-fill variable names based on total columns
    varrnd = ['var' + str(j+1).zfill(cc) for j in np.arange(0,cols)]

    if lock == False:
        print('showSubmitButton: False')
    
    # Test the boundaries of the inequalities
    for i in np.arange(0,rows):
        print(names[i] + ':'); print()
        
        print('ran = randomGenerator()')
        for j in np.arange(0,cols):
            print(varrnd[j] + ' = ran.float(-999,-100)')
        print()
        
        # Use incidence matrix to identify which variables occur in this inequality, and create lists accordingly
        varrndlist = ''; varstr = ''; varlist = ''; varilist = ''
        
        k = 0
        
        for j in np.arange(0,cols):
            if inc[i,j] == 1:
                if k > 0:
                    varrndlist += ','; varstr += ','; varlist += ','; varilist += ','    
                
                varrndlist += str(varrnd[j])
                varstr += str(variables[j])
                varlist += str(valueseq[i,j])
                varilist += str(valuesineq[i,j])
                k += 1
                    
        s = 'd0 = parseInequality(' + eqns[i] + ').differenceFunction('
        s += varstr + ').evaluateAt(' + varlist + ')'
        print(s)
        
        print('a0 = when d0 < 0 simpleFunction("-x","x").evaluateAt(d0) otherwise d0'); print()
        
        print('cor0 = when a0 <= ' + str(thr) + ' true otherwise false'); print()        
        
        s = 'd1 = parseInequality(this.latex).differenceFunction('
        s += varstr + ').evaluateAt(' + varlist + ')'
        print(s)
        
        print('a1 = when d1 < 0 simpleFunction("-x","x").evaluateAt(d1) otherwise d1'); print()
        
        print('cor1 = when a1 <= ' + str(thr) + ' true otherwise false'); print()

        s = 'd2 = parseInequality(this.latex).differenceFunction('
        s += varstr + ').evaluateAt(' + varrndlist + ')'
        print(s)
        
        print('a2 = when d2 < 0 simpleFunction("-x","x").evaluateAt(d2) otherwise d2'); print()
        
        print('cor2 = when a2 > ' + str(thr) + ' or isUndefined(a2) true otherwise false'); print()
        
        s = 'd3 = parseInequality(' + eqns[i] + ').differenceFunction('
        s += varstr + ').evaluateAt(' + varilist + ')'
        print(s)
        
        print('s3 = when d3 > 0 1 otherwise -1'); print()
        
        s = 'd4 = parseInequality(this.latex).differenceFunction('
        s += varstr + ').evaluateAt(' + varilist + ')'
        print(s)
        
        print('s4 = when d4 > 0 1 otherwise -1'); print()
        
        print('s34 = simpleFunction("a/b","a","b").evaluateAt(s3,s4)')
        print('cor34 = when s34 > 0 true otherwise false'); print()
        
        s = ('cors = when parseInequality(this.latex).isStrict false otherwise true')
        print(s); print()
        
        print('suffix: when cor0 and cor1 and cor2 and cor34 and cors "✅" otherwise "❌"'); print()
        print('correct: when cor0 and cor1 and cor2 and cor34 and cors true otherwise false'); print()

        if lock == True:
            print('disableEdit: when submit.pressCount > 1 true otherwise false'); print(); print()
        else:
            print('# disableEdit: when submit.pressCount > 1 true otherwise false'); print(); print()




import numpy as np

# Module to write Desmos code for reaction balancing exercise

def bal (rxn,name,elem,comp,var,coeff):
    # Useful strings based on reaction data
    
    varstr = ''; varlist = ''; nvarlist = ''
    
    for cf in np.arange(0,comp):
        varstr += '"' + var[cf] + '"'
        varlist += var[cf]
        nvarlist += var[cf] + var[cf]
        if cf != comp-1:
            varstr += ','; varlist += ','; nvarlist += ','
    
    # Intro component
    
    print(name + 'intro:'); print()
    
    print('rxn0 = ' + rxn); print()
    for cf in np.arange(0,comp):
        print(var[cf] + 'val = ' + str(coeff[cf]))
    print(); print()
    
    # Balance components
    for el in np.arange(0,elem):
        print(name + 'balance' + str(el+1) + ':'); print()
        
        for cf in np.arange(0,comp):
            print(var[cf] + ' = ' + name + 'intro.script.' + var[cf] + 'val')
        print()
         
        print('ran = randomGenerator()')
        for cf in np.arange(0,comp):
            print(var[cf] + var[cf], '= ran.float(-999,-100)')
        print()
        
        s = 'df1 = parseEquation(this.latex).differenceFunction('
        s += varstr + ').evaluateAt(' + varlist + ')'
        print(s)
        
        print('df2 = when df1 < 0 numericValue("-${df1}") otherwise df1'); print()
        
        print('cor1 = when df2 <= 0.01 true otherwise false'); print()
        
        s = 'df3 = parseEquation(this.latex).differenceFunction('
        s += varstr + ').evaluateAt(' + nvarlist + ')'
        print(s)
        
        print('df4 = when df3 < 0 numericValue("-${df3}") otherwise df3'); print()
        
        print('cor2 = when df4 > 0.01 or isUndefined(df4) true otherwise false'); print()
        
        print('suffix: when cor1 and cor2 "✅" otherwise "❌"'); print()
        
        print('correct: when cor1 and cor2 true otherwise false'); print()
        
        print('disableEdit: when submit.pressCount > 1 true otherwise false'); print(); print()
        
    # Balance submit component
    
    print(name + 'balancesubmit:'); print()
    
    submitstr = ''
    for cf in np.arange(0,comp):
        submitstr += name + 'balance' + str(cf+1) + '.submitted'
        if cf != comp-1:
            submitstr += ' and '
    print('submit = when ' + submitstr + ' true otherwise false'); print(); print()
    
    # Final component
    
    print(name + 'final:'); print()
    
    print('rxn0 = ' + rxn); print()    
    for cf in np.arange(0,comp):
        s = 'rxn' + str(cf+1) + ' = substituteLatexVariable(rxn' + str(cf) + ','
        s += '`' + var[cf] + '`,' + name + 'table.script.' + var[cf] + ')'
        print(s)
    print()
    
    print('fb = when ' + name + 'table.script.cor "✅" otherwise "❌"')
    print(); print()
    
    # Table component
    
    print(name + 'table:'); print()
    
    print('maxrows = '+ str(comp)); print()
   
    for cf in np.arange(0,comp):
        scf = str(cf+1)
        print('cellContent(' + scf + ',1): "' + var[cf] + '"')
    print()
    
    for cf in np.arange(0,comp):
        scf = str(cf+1)
        s = 'df' + scf + ' = numericValue("${this.cellNumericValue(' + scf
        s += ',2)}-${' + name + 'intro.script.' + var[cf] + 'val}")'
        print(s)
        s = 'df' + scf + 'a = when df' + scf + ' >= 0 df' + scf
        s += ' otherwise numericValue("-${df' + scf + '}")'
        print(s)
    print()
    
    for cf in np.arange(0,comp):
        scf = str(cf+1)
        s = 'cor' + scf + ' = when df' + scf + 'a < 0.01 true otherwise false'
        print(s)
    print()
        
    for cf in np.arange(0,comp):
        scf = str(cf+1)
        print('cellSuffix(' + scf + ',2): when cor' + scf + ' "✅" otherwise "❌"')
    print()
    
    for cf in np.arange(0,comp):
        scf = str(cf+1); vcf = var[cf]
        print(vcf + ' = when this.cellContent(' + scf + ',2) = "" `' + vcf + '` otherwise')
        print('  when this.cellNumericValue(' + scf + ',2) = 1 `` otherwise')
        s = '    when this.cellNumericValue(' + scf + ',2) > 0 `${numericValue("\\round(${this.cellNumericValue('
        s += scf + ',2)},2)")}` otherwise `' + vcf + '`'
        print(s); print()
    
    s = 'cor = '
    for cf in np.arange(0,comp):
        scf = str(cf+1)
        s += 'cor' + scf
        if cf != comp-1:
            s += ' and '
    print(s)
    print()
    
    print('correct: cor')
    print()
    
    for cf in np.arange(0,comp):
        scf = str(cf+1)
        print('cellEditable(' + scf + ',2): when submit.pressCount <= 1 true otherwise false')



import numpy as np

# Module to write Desmos code for checking equations

def equation (names,eqns,variables,values,thr=0.01,exact=0,inc=[[]],lock=True):
    # The vectors names and eqns must be of the same size
    # The vectors variables and values must be of the same size
    # If equations have to be matched exactly (exact == 1):
    #   the incidence matrix inc must be sized with equations as rows and variables as columns
    #   this matrix must contain only 0s and 1s

    import sys
    sys.stdin.reconfigure(encoding='utf-8')
    sys.stdout.reconfigure(encoding='utf-8')
    
    rows = np.size(names,0)
    cols = np.size(variables,0)
    
    cc = int(np.floor(np.log(cols)/np.log(10))+1) # zero-fill variable names based on total columns
    varrnd = ['var' + str(j+1).zfill(cc) for j in np.arange(0,cols)]
    
    # Equations do not have to be exactly matched; create lists outside loop
    if exact == 0: 
        varrndlist = str(varrnd).replace('[','').replace(']','').replace("'",'')   
        varstr = str(variables).replace('[','').replace(']','').replace("'",'')        
        varlist = str(values).replace('[','').replace(']','')
        varlist = varlist.replace("'",'') # remove double quotes from any strings

    if lock == False:
        print('showSubmitButton: False')

    for i in np.arange(0,rows):
        print(names[i] + ':'); print()
        
        print('ran = randomGenerator()')
        for j in np.arange(0,cols):
            print(varrnd[j] + ' = ran.float(-999,-100)')
        print()
        
        if exact == 0:        
            s = 'd1 = parseEquation(this.latex).differenceFunction('
            s += varstr + ').evaluateAt(' + varlist + ')'
            print(s)
            
            print('a1 = when d1 < 0 simpleFunction("-x","x").evaluateAt(d1) otherwise d1'); print()
            
            print('cor1 = when a1 <= ' + str(thr) + ' true otherwise false'); print()
    
            s = 'd2 = parseEquation(this.latex).differenceFunction('
            s += varstr + ').evaluateAt(' + varrndlist + ')'
            print(s)
            
            print('a2 = when d2 < 0 simpleFunction("-x","x").evaluateAt(d2) otherwise d2'); print()
            
            print('cor2 = when a2 > ' + str(thr) + ' or isUndefined(a2) true otherwise false'); print()
            
            print('suffix: when cor1 and cor2 "✅" otherwise "❌"'); print()
            print('correct: when cor1 and cor2 true otherwise false'); print()
        
        elif exact == 1:
            # Use incidence matrix to identify which variables occur in this equation, and create lists accordingly
            varrndlist = ''; varstr = ''; varlist = ''
            k = 0
            for j in np.arange(0,cols):
                if inc[i,j] == 1:
                    if k > 0:
                        varrndlist += ','; varstr += ','; varlist += ','                 
                    varrndlist += str(varrnd[j])
                    varstr += str(variables[j])
                    varlist += str(values[j])
                    k += 1
                        
            s = 'd0 = parseEquation(' + eqns[i] + ').differenceFunction('
            s += varstr + ').evaluateAt(' + varlist + ')'
            print(s)
            
            print('a0 = when d0 < 0 simpleFunction("-x","x").evaluateAt(d0) otherwise d0'); print()
            
            print('cor0 = when a0 <= ' + str(thr) + ' true otherwise false'); print()        
            
            s = 'd1 = parseEquation(this.latex).differenceFunction('
            s += varstr + ').evaluateAt(' + varlist + ')'
            print(s)
            
            print('a1 = when d1 < 0 simpleFunction("-x","x").evaluateAt(d1) otherwise d1'); print()
            
            print('cor1 = when a1 <= ' + str(thr) + ' true otherwise false'); print()
    
            s = 'd2 = parseEquation(this.latex).differenceFunction('
            s += varstr + ').evaluateAt(' + varrndlist + ')'
            print(s)
            
            print('a2 = when d2 < 0 simpleFunction("-x","x").evaluateAt(d2) otherwise d2'); print()
            
            print('cor2 = when a2 > ' + str(thr) + ' or isUndefined(a2) true otherwise false'); print()
            
            print('suffix: when cor0 and cor1 and cor2 "✅" otherwise "❌"'); print()
            print('correct: when cor0 and cor1 and cor2 true otherwise false'); print()

        if lock == True:
            print('disableEdit: when submit.pressCount > 1 true otherwise false'); print(); print()
        else:
            print('# disableEdit: when submit.pressCount > 1 true otherwise false'); print(); print()
